import React, { useState, useEffect, useRef, useCallback } from "react";
import { AlertCircle, Plus, X, Image as ImageIcon } from "lucide-react";
import { useTranslation } from "react-i18next";
import { pinchoService, establishmentService } from "../services";
import { Pincho, Establishment } from "../types/domain";
import { useAuth } from "../store/AuthContext";
import "./PinchosList.css";
import Loader from "../components/Loader";
import { resizeImage, isValidImage, isWithinSizeLimit } from "../utils/imageUtils";
import PinchoCard from "../components/PinchoCard";

const PinchosList = ({ showCreateModal = false }: { showCreateModal?: boolean }) => {
  const { t } = useTranslation();
  const { isAuthenticated } = useAuth();
  const [pinchos, setPinchos] = useState<Pincho[]>([]);
  const [establishments, setEstablishments] = useState<Establishment[]>([]);
  const [loading, setLoading] = useState(true);
  const [error, setError] = useState<string | null>(null);
  const [isModalOpen, setModalOpen] = useState(false);
  const [isNewEstablishmentFormOpen, setNewEstablishmentFormOpen] =
    useState(false);
  const [formSubmitting, setFormSubmitting] = useState(false);
  const [formErrors, setFormErrors] = useState({
    name: '',
    price: '',
    establishment_id: '',
    image: ''
  });
  const [imagePreview, setImagePreview] = useState<string | null>(null);
  const modalRef = useRef<HTMLDivElement>(null);
  
  const [newPincho, setNewPincho] = useState({
    name: "",
    description: "",
    price: "",
    establishment_id: "",
    image: null as File | null
  });
  const [newEstablishment, setNewEstablishment] = useState({
    name: "",
    address: ""
  });

  useEffect(() => {
    const loadPinchosAndEstablishments = async () => {
      try {
        const [pinchosData, establishmentsData] = await Promise.all([
          pinchoService.getPinchosList(),
          establishmentService.getAllEstablishments()
        ]);
        setPinchos(pinchosData);
        setEstablishments(establishmentsData);
      } catch (err: unknown) {
        setError((err as Error).message || t("error.generic"));
      } finally {
        setLoading(false);
      }
    };

    loadPinchosAndEstablishments();
  }, [t]);

  useEffect(() => {
    if (showCreateModal) {
      setModalOpen(true);
    }
  }, [showCreateModal]);
  
  // Manejar cierre con Escape y click fuera del modal
  useEffect(() => {
    const handleEscape = (e: KeyboardEvent) => {
      if (e.key === 'Escape' && (isModalOpen || isNewEstablishmentFormOpen)) {
        if (isNewEstablishmentFormOpen) {
          setNewEstablishmentFormOpen(false);
        } else {
          setModalOpen(false);
        }
      }
    };
    
    const handleClickOutside = (e: MouseEvent) => {
      if (modalRef.current && !modalRef.current.contains(e.target as Node)) {
        if (isNewEstablishmentFormOpen) {
          setNewEstablishmentFormOpen(false);
        } else {
          setModalOpen(false);
        }
      }
    };
    
    document.addEventListener('keydown', handleEscape);
    document.addEventListener('mousedown', handleClickOutside);
    
    return () => {
      document.removeEventListener('keydown', handleEscape);
      document.removeEventListener('mousedown', handleClickOutside);
    };
  }, [isModalOpen, isNewEstablishmentFormOpen]);

  // Función de validación
  const validateForm = useCallback(() => {
    const errors = {
      name: '',
      price: '',
      establishment_id: '',
      image: ''
    };
    let isValid = true;
    
    // Validar nombre
    if (!newPincho.name.trim()) {
      errors.name = t("pinchosList.validation.nameRequired");
      isValid = false;
    }
    
    // Validar establecimiento
    if (!newPincho.establishment_id) {
      errors.establishment_id = t("pinchosList.validation.establishmentRequired");
      isValid = false;
    }
    
    // Validar precio
    const price = parseFloat(newPincho.price);
    if (isNaN(price) || price <= 0) {
      errors.price = t("pinchosList.validation.priceInvalid");
      isValid = false;
    }
    
    // Validar imagen si se ha seleccionado una
    if (newPincho.image) {
      if (!isValidImage(newPincho.image)) {
        errors.image = t("pinchosList.validation.imageTypeInvalid");
        isValid = false;
      } else if (!isWithinSizeLimit(newPincho.image)) {
        errors.image = t("pinchosList.validation.imageSizeExceeded");
        isValid = false;
      }
    }
    
    setFormErrors(errors);
    return isValid;
  }, [newPincho, t]);

  const handleFileChange = async (e: React.ChangeEvent<HTMLInputElement>) => {
    const files = e.target.files;
    if (!files || files.length === 0) {
      setNewPincho(prev => ({ ...prev, image: null }));
      setImagePreview(null);
      return;
    }
    
    const selectedFile = files[0];
    
    // Validar el tipo de archivo
    if (!isValidImage(selectedFile)) {
      setFormErrors(prev => ({
        ...prev,
        image: t("pinchosList.validation.imageTypeInvalid")
      }));
      setNewPincho(prev => ({ ...prev, image: null }));
      setImagePreview(null);
      return;
    }
    
    // Validar el tamaño
    if (!isWithinSizeLimit(selectedFile)) {
      setFormErrors(prev => ({
        ...prev,
        image: t("pinchosList.validation.imageSizeExceeded")
      }));
      setNewPincho(prev => ({ ...prev, image: null }));
      setImagePreview(null);
      return;
    }
    
    try {
      // Redimensionar imagen antes de guardarla
      const optimizedImage = await resizeImage(selectedFile);
      
      // Actualizar el estado con la imagen optimizada
      setNewPincho(prev => ({
        ...prev,
        image: optimizedImage
      }));
      
      // Crear preview de la imagen
      const previewUrl = URL.createObjectURL(optimizedImage);
      setImagePreview(previewUrl);
      
      // Limpiar el error si existe
      if (formErrors.image) {
        setFormErrors(prev => ({ ...prev, image: '' }));
      }
    } catch (error) {
      console.error("Error al procesar la imagen:", error);
      setFormErrors(prev => ({
        ...prev,
        image: t("pinchosList.validation.imageProcessingError")
      }));
    }
  };

  // Limpiar la URL del objeto creado cuando se cierre el modal o se envíe el formulario
  useEffect(() => {
    return () => {
      if (imagePreview) {
        URL.revokeObjectURL(imagePreview);
      }
    };
  }, [imagePreview]);

  const handleSubmit = async (e: React.FormEvent) => {
    e.preventDefault();
    
    // Validar formulario
    if (!validateForm()) {
      return;
    }
    
    setFormSubmitting(true);
    
    try {
      // Convertir el precio a número
      const priceAsNumber = parseFloat(newPincho.price);
      
      // Verificar que tenemos los campos requeridos
      if (!newPincho.name || !newPincho.establishment_id) {
        throw new Error('Los campos nombre y establecimiento son obligatorios');
      }
      
      // Preparar el FormData
      const formData = new FormData();
      
      // Añadir campos obligatorios
      formData.append('name', newPincho.name.trim());
      formData.append('establishment_id', newPincho.establishment_id);
      
      // Añadir campos opcionales
      if (newPincho.description) {
        formData.append('description', newPincho.description);
      }
      
      // Añadir el precio (asegurarnos de que es un número válido)
      if (!isNaN(priceAsNumber) && priceAsNumber > 0) {
        formData.append('price', priceAsNumber.toString());
      } else {
        throw new Error('El precio debe ser un número mayor que cero');
      }
      
      // Si hay una imagen, añadirla al FormData
      if (newPincho.image) {
        formData.append('image', newPincho.image);
      }
      
      // Enviar los datos al servidor
      const createdPincho = await pinchoService.createPinchoWithImage(formData);
      
      // Añadir el nuevo pincho al listado
      setPinchos((prevPinchos) => [...prevPinchos, createdPincho]);
      
      // Limpiar el formulario
      setNewPincho({
        name: "",
        description: "",
        price: "",
        establishment_id: "",
        image: null
      });
      
      // Limpiar la vista previa de la imagen
      if (imagePreview) {
        URL.revokeObjectURL(imagePreview);
        setImagePreview(null);
      }
      
      // Cerrar el modal
      setModalOpen(false);
      
      // Informar al usuario
      alert(t("pinchosList.createSuccess"));
    } catch (error: unknown) {
      console.error("Error creando el pincho:", error);
      const errorMessage = error instanceof Error ? error.message : t("pinchosList.createError");
      alert(errorMessage);
    } finally {
      setFormSubmitting(false);
    }
  };

  const handleNewEstablishmentSubmit = async (e: React.FormEvent) => {
    e.preventDefault();
    setFormSubmitting(true);
    
    try {
      const newEstablishmentData = await establishmentService.createEstablishment({
        name: newEstablishment.name,
        address: newEstablishment.address
      });
      
      if (newEstablishmentData) {
        // Añadir el nuevo establecimiento a la lista de establecimientos
        setEstablishments((prevEstablishments) => [
          ...prevEstablishments,
          newEstablishmentData
        ]);
        // Actualizar el estado de newPincho para seleccionar automáticamente el nuevo establecimiento
        setNewPincho((prevPincho) => ({
          ...prevPincho,
          establishment_id: newEstablishmentData.id
        }));
        // Cerrar el formulario de establecimiento y volver al de pincho
        setNewEstablishmentFormOpen(false);
      }
    } catch (error) {
      console.error("Error creando el establecimiento:", error);
      alert(t("establishmentForm.error") || "Error al crear el establecimiento");
    } finally {
      setFormSubmitting(false);
    }
  };

  const handleChange = (
    e: React.ChangeEvent<
      HTMLInputElement | HTMLTextAreaElement | HTMLSelectElement
    >
  ) => {
    const { name, value } = e.target;
    
    if (isNewEstablishmentFormOpen) {
      setNewEstablishment((prev) => ({ ...prev, [name]: value }));
    } else {
      setNewPincho((prev) => ({ ...prev, [name]: value }));
      
      // Limpiar el error asociado con este campo si existe
      if (formErrors[name as keyof typeof formErrors]) {
        setFormErrors(prev => ({
          ...prev,
          [name]: ''
        }));
      }
    }
  };

  if (loading) {
    return (
      <div className="container pinchos">
        <div className="pinchos__header">
          <h1 className="pinchos__title">
            {t("pinchosList.loading")}
          </h1>
        </div>
        <div className="pinchos__grid">
          {Array.from({ length: 6 }).map((_, index) => (
            <div key={index} className="pinchos__card-skeleton-wrapper">
              <Loader 
                variant="card-skeleton" 
                size="medium" 
                count={1}
              />
            </div>
          ))}
        </div>
      </div>
    );
  }

  if (error) {
    return (
      <div className="pinchos__error">
        <AlertCircle size={48} className="pinchos__error-icon" />
        <h2>{t("error.title")}</h2>
        <p>{t("error.message")} {error}</p>
      </div>
    );
  }

  return (
    <div className="container pinchos">
      <div className="pinchos__header">
        <h1 className="pinchos__title">{t("pinchosList.title")}</h1>
        {isAuthenticated && (
          <button
            className="button button-primary pinchos__add-button"
            onClick={() => setModalOpen(true)}
          >
            <Plus size={20} />
            {t("pinchosList.addPincho")}
          </button>
        )}
      </div>

      <div className="pinchos__grid">
        {pinchos.map((pincho) => (
          <PinchoCard key={pincho.id} pincho={pincho} />
        ))}
      </div>

      {isModalOpen && !isNewEstablishmentFormOpen && (
        <div className="pinchos__modal-overlay" aria-modal="true" role="dialog">
          <div className="pinchos__modal" ref={modalRef}>
            <div className="pinchos__modal-header">
              <h2 className="pinchos__modal-title">{t("pinchosList.addPincho")}</h2>
              <button
                onClick={() => setModalOpen(false)}
                className="pinchos__modal-close"
                aria-label="Cerrar"
              >
                <X size={24} />
              </button>
            </div>

            <form onSubmit={handleSubmit}>
              <div className="pinchos__form-group">
                <label className="pinchos__form-label" htmlFor="pincho-name">{t("pinchosList.form.name")}</label>
                <input
                  type="text"
                  id="pincho-name"
                  name="name"
                  value={newPincho.name}
                  onChange={handleChange}
                  className={`pinchos__input ${formErrors.name ? 'is-error' : ''}`}
                  required
                  aria-invalid={!!formErrors.name}
                  aria-describedby={formErrors.name ? "name-error" : undefined}
                />
                {formErrors.name && (
                  <div className="pinchos__error-message" id="name-error">
                    {formErrors.name}
                  </div>
                )}
              </div>
              <div className="pinchos__form-group">
                <label className="pinchos__form-label" htmlFor="pincho-description">{t("pinchosList.form.description")}</label>
                <textarea
                  id="pincho-description"
                  name="description"
                  value={newPincho.description}
                  onChange={handleChange}
                  className="pinchos__input pinchos__textarea"
                  rows={4}
                  required
                />
              </div>
              <div className="pinchos__form-group">
                <label className="pinchos__form-label" htmlFor="pincho-price">{t("pinchosList.form.price")}</label>
                <input
                  type="number"
                  id="pincho-price"
                  name="price"
                  value={newPincho.price}
                  onChange={handleChange}
                  className={`pinchos__input ${formErrors.price ? 'is-error' : ''}`}
                  required
                  step="0.01"
                  min="0.01"
                  aria-invalid={!!formErrors.price}
                  aria-describedby={formErrors.price ? "price-error" : undefined}
                />
                {formErrors.price && (
                  <div className="pinchos__error-message" id="price-error">
                    {formErrors.price}
                  </div>
                )}
              </div>
              <div className="pinchos__form-group">
                <label className="pinchos__form-label" htmlFor="pincho-image">{t("pinchosList.form.image")}</label>
                <input
                  type="file"
                  id="pincho-image"
                  name="image"
                  onChange={handleFileChange}
                  className={`pinchos__input pinchos__file-input ${formErrors.image ? 'is-error' : ''}`}
                  accept="image/jpeg,image/jpg,image/png,image/gif,image/webp"
                  aria-invalid={!!formErrors.image}
                  aria-describedby={formErrors.image ? "image-error" : undefined}
                />
                <small className="pinchos__form-help">
                  {t("pinchosList.form.imageHelp")}
                </small>
                
                {formErrors.image && (
                  <div className="pinchos__error-message" id="image-error">
                    {formErrors.image}
                  </div>
                )}
                
                {imagePreview && (
                  <div className="pinchos__image-preview">
                    <img 
                      src={imagePreview} 
                      alt={t("pinchosList.form.imagePreview")} 
                      className="pinchos__preview-img" 
                    />
                  </div>
                )}
                
                {!imagePreview && (
                  <div className="pinchos__no-image">
                    <ImageIcon size={48} />
                    <p>{t("pinchosList.form.noImage")}</p>
                  </div>
                )}
              </div>
              <div className="pinchos__form-group">
                <label className="pinchos__form-label" htmlFor="pincho-establishment">{t("pinchosList.form.establishment")}</label>
                <div className="pinchos__select-container">
                  <select
                    id="pincho-establishment"
                    name="establishment_id"
                    value={newPincho.establishment_id}
                    onChange={handleChange}
                    className={`pinchos__input ${formErrors.establishment_id ? 'is-error' : ''}`}
                    required
                    aria-invalid={!!formErrors.establishment_id}
                    aria-describedby={formErrors.establishment_id ? "establishment-error" : undefined}
                  >
                    <option value="">
                      {t("pinchosList.form.selectEstablishment")}
                    </option>
                    {establishments.map((establishment) => (
                      <option key={establishment.id} value={establishment.id}>
                        {establishment.name}
                      </option>
                    ))}
                  </select>
                  <button
                    type="button"
                    onClick={() => setNewEstablishmentFormOpen(true)}
                    className="button"
                  >
                    {t("pinchosList.form.createEstablishment")}
                  </button>
                </div>
                {formErrors.establishment_id && (
                  <div className="pinchos__error-message" id="establishment-error">
                    {formErrors.establishment_id}
                  </div>
                )}
              </div>
              <button 
                type="submit" 
                className="button button-primary" 
                disabled={formSubmitting}
              >
                {formSubmitting ? t("loading") : t("pinchosList.form.save")}
              </button>
            </form>
          </div>
        </div>
      )}

      {isNewEstablishmentFormOpen && (
        <div className="pinchos__modal-overlay" aria-modal="true" role="dialog">
          <div className="pinchos__modal" ref={modalRef}>
            <div className="pinchos__modal-header">
              <h2 className="pinchos__modal-title">{t("pinchosList.form.createEstablishment")}</h2>
              <button
                onClick={() => setNewEstablishmentFormOpen(false)}
                className="pinchos__modal-close"
                aria-label="Cerrar"
              >
                <X size={24} />
              </button>
            </div>

            <form onSubmit={handleNewEstablishmentSubmit}>
              <div className="pinchos__form-group">
                <label className="pinchos__form-label" htmlFor="establishment-name">
                  {t("establishmentForm.name")} *
                </label>
                <input
                  type="text"
                  id="establishment-name"
                  name="name"
                  value={newEstablishment.name}
                  onChange={handleChange}
                  placeholder={t("establishmentForm.placeholders.name")}
                  className="pinchos__input"
                  required
                />
              </div>
              <div className="pinchos__form-group">
                <label className="pinchos__form-label" htmlFor="establishment-address">
                  {t("establishmentForm.address")} *
                </label>
                <input
                  type="text"
                  id="establishment-address"
                  name="address"
                  value={newEstablishment.address}
                  onChange={handleChange}
                  placeholder={t("establishmentForm.placeholders.address")}
                  className="pinchos__input"
                  required
                />
              </div>
              <button 
                type="submit" 
                className="button button-primary"
                disabled={formSubmitting}
              >
                {formSubmitting ? t("loading") : t("establishmentForm.save")}
              </button>
            </form>
          </div>
        </div>
      )}
    </div>
  );
};

export default PinchosList;
